@@ -204,7 +204,7 @@ write_buf(
 			print_warning(
 			    "obfuscation corrupted block at %s bno 0x%llx/0x%x",
 				bp->b_ops->name,
-				(long long)bp->b_bn, bp->b_bcount);
+				(long long)xfs_buf_daddr(bp), BBTOB(bp->b_length));
 		}
 	}
 
@@ -487,7 +487,7 @@ copy_free_bno_btree(
 					"root in agf %u", root, agno);
 		return 1;
 	}
-	if (levels >= XFS_BTREE_MAXLEVELS) {
+	if (levels > mp->m_alloc_maxlevels) {
 		if (show_warnings)
 			print_warning("invalid level (%u) in bnobt root "
 					"in agf %u", levels, agno);
@@ -515,7 +515,7 @@ copy_free_cnt_btree(
 					"root in agf %u", root, agno);
 		return 1;
 	}
-	if (levels >= XFS_BTREE_MAXLEVELS) {
+	if (levels > mp->m_alloc_maxlevels) {
 		if (show_warnings)
 			print_warning("invalid level (%u) in cntbt root "
 					"in agf %u", levels, agno);
@@ -574,7 +574,7 @@ copy_rmap_btree(
 	xfs_agblock_t	root;
 	int		levels;
 
-	if (!xfs_sb_version_hasrmapbt(&mp->m_sb))
+	if (!xfs_has_rmapbt(mp))
 		return 1;
 
 	root = be32_to_cpu(agf->agf_roots[XFS_BTNUM_RMAP]);
@@ -587,7 +587,7 @@ copy_rmap_btree(
 					"root in agf %u", root, agno);
 		return 1;
 	}
-	if (levels >= XFS_BTREE_MAXLEVELS) {
+	if (levels > mp->m_rmap_maxlevels) {
 		if (show_warnings)
 			print_warning("invalid level (%u) in rmapbt root "
 					"in agf %u", levels, agno);
@@ -646,7 +646,7 @@ copy_refcount_btree(
 	xfs_agblock_t	root;
 	int		levels;
 
-	if (!xfs_sb_version_hasreflink(&mp->m_sb))
+	if (!xfs_has_reflink(mp))
 		return 1;
 
 	root = be32_to_cpu(agf->agf_refcount_root);
@@ -659,7 +659,7 @@ copy_refcount_btree(
 					"root in agf %u", root, agno);
 		return 1;
 	}
-	if (levels >= XFS_BTREE_MAXLEVELS) {
+	if (levels > mp->m_refc_maxlevels) {
 		if (show_warnings)
 			print_warning("invalid level (%u) in refcntbt root "
 					"in agf %u", levels, agno);
@@ -1234,7 +1234,7 @@ generate_obfuscated_name(
 
 static void
 process_sf_dir(
-	xfs_dinode_t		*dip)
+	struct xfs_dinode	*dip)
 {
 	struct xfs_dir2_sf_hdr	*sfp;
 	xfs_dir2_sf_entry_t	*sfep;
@@ -1339,7 +1339,7 @@ obfuscate_path_components(
 
 static void
 process_sf_symlink(
-	xfs_dinode_t		*dip)
+	struct xfs_dinode	*dip)
 {
 	uint64_t		len;
 	char			*buf;
@@ -1363,19 +1363,19 @@ process_sf_symlink(
 
 static void
 process_sf_attr(
-	xfs_dinode_t		*dip)
+	struct xfs_dinode		*dip)
 {
 	/*
 	 * with extended attributes, obfuscate the names and fill the actual
 	 * values with 'v' (to see a valid string length, as opposed to NULLs)
 	 */
 
-	xfs_attr_shortform_t	*asfp;
-	xfs_attr_sf_entry_t	*asfep;
-	int			ino_attr_size;
-	int			i;
+	struct xfs_attr_shortform	*asfp;
+	struct xfs_attr_sf_entry	*asfep;
+	int				ino_attr_size;
+	int				i;
 
-	asfp = (xfs_attr_shortform_t *)XFS_DFORK_APTR(dip);
+	asfp = (struct xfs_attr_shortform *)XFS_DFORK_APTR(dip);
 	if (asfp->hdr.count == 0)
 		return;
 
@@ -1399,7 +1399,7 @@ process_sf_attr(
 						"%llu", (long long)cur_ino);
 			break;
 		} else if ((char *)asfep - (char *)asfp +
-				XFS_ATTR_SF_ENTSIZE(asfep) > ino_attr_size) {
+				xfs_attr_sf_entsize(asfep) > ino_attr_size) {
 			if (show_warnings)
 				print_warning("attr entry length in inode %llu "
 					"overflows space", (long long)cur_ino);
@@ -1413,8 +1413,8 @@ process_sf_attr(
 			       asfep->valuelen);
 		}
 
-		asfep = (xfs_attr_sf_entry_t *)((char *)asfep +
-				XFS_ATTR_SF_ENTSIZE(asfep));
+		asfep = (struct xfs_attr_sf_entry *)((char *)asfep +
+				xfs_attr_sf_entsize(asfep));
 	}
 
 	/* zero stale data in rest of space in attr fork, if any */
@@ -1536,13 +1536,13 @@ process_dir_data_block(
 			blp = (xfs_dir2_leaf_entry_t *)btp;
 
 		end_of_data = (char *)blp - block;
-		if (xfs_sb_version_hascrc(&mp->m_sb))
+		if (xfs_has_crc(mp))
 			wantmagic = XFS_DIR3_BLOCK_MAGIC;
 		else
 			wantmagic = XFS_DIR2_BLOCK_MAGIC;
 	} else { /* leaf/node format */
 		end_of_data = mp->m_dir_geo->fsbcount << mp->m_sb.sb_blocklog;
-		if (xfs_sb_version_hascrc(&mp->m_sb))
+		if (xfs_has_crc(mp))
 			wantmagic = XFS_DIR3_DATA_MAGIC;
 		else
 			wantmagic = XFS_DIR2_DATA_MAGIC;
@@ -1664,7 +1664,7 @@ process_symlink_block(
 	}
 	link = iocur_top->data;
 
-	if (xfs_sb_version_hascrc(&(mp)->m_sb))
+	if (xfs_has_crc((mp)))
 		link += sizeof(struct xfs_dsymlink_hdr);
 
 	if (obfuscate)
@@ -1675,7 +1675,7 @@ process_symlink_block(
 
 		linklen = strlen(link);
 		zlen = mp->m_sb.sb_blocksize - linklen;
-		if (xfs_sb_version_hascrc(&mp->m_sb))
+		if (xfs_has_crc(mp))
 			zlen -= sizeof(struct xfs_dsymlink_hdr);
 		if (zlen < mp->m_sb.sb_blocksize)
 			memset(link + linklen, 0, zlen);
@@ -2193,7 +2193,7 @@ scanfunc_bmap(
 
 static int
 process_btinode(
-	xfs_dinode_t 		*dip,
+	struct xfs_dinode 	*dip,
 	typnm_t			itype)
 {
 	xfs_bmdr_block_t	*dib;
@@ -2273,7 +2273,7 @@ process_btinode(
 
 static int
 process_exinode(
-	xfs_dinode_t 		*dip,
+	struct xfs_dinode 	*dip,
 	typnm_t			itype)
 {
 	int			whichfork;
@@ -2303,7 +2303,7 @@ process_exinode(
 
 static int
 process_inode_data(
-	xfs_dinode_t		*dip,
+	struct xfs_dinode	*dip,
 	typnm_t			itype)
 {
 	switch (dip->di_format) {
@@ -2333,7 +2333,7 @@ process_inode_data(
 
 static int
 process_dev_inode(
-	xfs_dinode_t		*dip)
+	struct xfs_dinode		*dip)
 {
 	if (XFS_DFORK_NEXTENTS(dip, XFS_DATA_FORK)) {
 		if (show_warnings)
@@ -2362,7 +2362,7 @@ static int
 process_inode(
 	xfs_agnumber_t		agno,
 	xfs_agino_t 		agino,
-	xfs_dinode_t 		*dip,
+	struct xfs_dinode 	*dip,
 	bool			free_inode)
 {
 	int			success;
@@ -2476,7 +2476,7 @@ copy_inode_chunk(
 	 * Also make sure that that we don't process more than the single record
 	 * we've been passed (large block sizes can hold multiple inode chunks).
 	 */
-	if (xfs_sb_version_hassparseinodes(&mp->m_sb))
+	if (xfs_has_sparseinodes(mp))
 		blks_per_buf = igeo->blocks_per_cluster;
 	else
 		blks_per_buf = igeo->ialloc_blks;
@@ -2509,7 +2509,7 @@ copy_inode_chunk(
 	if ((mp->m_sb.sb_inopblock <= XFS_INODES_PER_CHUNK && off != 0) ||
 			(mp->m_sb.sb_inopblock > XFS_INODES_PER_CHUNK &&
 					off % XFS_INODES_PER_CHUNK != 0) ||
-			(xfs_sb_version_hasalign(&mp->m_sb) &&
+			(xfs_has_align(mp) &&
 					mp->m_sb.sb_inoalignmt != 0 &&
 					agbno % mp->m_sb.sb_inoalignmt != 0)) {
 		if (show_warnings)
@@ -2534,9 +2534,9 @@ copy_inode_chunk(
 		}
 
 		for (i = 0; i < inodes_per_buf; i++) {
-			xfs_dinode_t	*dip;
+			struct xfs_dinode	*dip;
 
-			dip = (xfs_dinode_t *)((char *)iocur_top->data +
+			dip = (struct xfs_dinode *)((char *)iocur_top->data +
 					((off + i) << mp->m_sb.sb_inodelog));
 
 			/* process_inode handles free inodes, too */
@@ -2650,7 +2650,7 @@ copy_inodes(
 					"root in agi %u", root, agno);
 		return 1;
 	}
-	if (levels >= XFS_BTREE_MAXLEVELS) {
+	if (levels > M_IGEO(mp)->inobt_maxlevels) {
 		if (show_warnings)
 			print_warning("invalid level (%u) in inobt root "
 					"in agi %u", levels, agno);
@@ -2660,7 +2660,7 @@ copy_inodes(
 	if (!scan_btree(agno, root, levels, TYP_INOBT, &finobt, scanfunc_ino))
 		return 0;
 
-	if (xfs_sb_version_hasfinobt(&mp->m_sb)) {
+	if (xfs_has_finobt(mp)) {
 		root = be32_to_cpu(agi->agi_free_root);
 		levels = be32_to_cpu(agi->agi_free_level);
 
@@ -2891,8 +2891,8 @@ copy_log(void)
 
 		logstart = XFS_FSB_TO_DADDR(mp, mp->m_sb.sb_logstart);
 		logblocks = XFS_FSB_TO_BB(mp, mp->m_sb.sb_logblocks);
-		logversion = xfs_sb_version_haslogv2(&mp->m_sb) ? 2 : 1;
-		if (xfs_sb_version_hascrc(&mp->m_sb))
+		logversion = xfs_has_logv2(mp) ? 2 : 1;
+		if (xfs_has_crc(mp))
 			cycle = log.l_curr_cycle + 1;
 
 		libxfs_log_clear(NULL, iocur_top->data, logstart, logblocks,
