@@ -16,7 +16,7 @@ xlog_print_find_oldest(
 	struct xlog	*log,
 	xfs_daddr_t	*last_blk)
 {
-	xfs_buf_t	*bp;
+	struct xfs_buf	*bp;
 	xfs_daddr_t	first_blk;
 	uint		first_half_cycle, last_half_cycle;
 	int		error = 0;
@@ -39,7 +39,7 @@ xlog_print_find_oldest(
 		error = xlog_find_cycle_start(log, bp, first_blk,
 					      last_blk, last_half_cycle);
 
-	xlog_put_bp(bp);
+	libxfs_buf_relse(bp);
 	return error;
 }
 
@@ -68,7 +68,7 @@ xlog_recover_print_data(
 
 STATIC void
 xlog_recover_print_buffer(
-	xlog_recover_item_t	*item)
+	struct xlog_recover_item *item)
 {
 	xfs_agi_t		*agi;
 	xfs_agf_t		*agf;
@@ -76,7 +76,7 @@ xlog_recover_print_buffer(
 	char			*p;
 	int			len, num, i;
 	xfs_daddr_t		blkno;
-	xfs_disk_dquot_t	*ddq;
+	struct xfs_disk_dquot	*ddq;
 
 	f = (xfs_buf_log_format_t *)item->ri_buf[0].i_addr;
 	printf("	");
@@ -165,14 +165,14 @@ xlog_recover_print_buffer(
 				be32_to_cpu(agf->agf_freeblks),
 				be32_to_cpu(agf->agf_longest));
 		} else if (*(uint *)p == XFS_DQUOT_MAGIC) {
-			ddq = (xfs_disk_dquot_t *)p;
+			ddq = (struct xfs_disk_dquot *)p;
 			printf(_("	DQUOT Buffer:\n"));
 			if (!print_buffer)
 				continue;
 			printf(_("		UIDs 0x%lx-0x%lx\n"),
 			       (unsigned long)be32_to_cpu(ddq->d_id),
 			       (unsigned long)be32_to_cpu(ddq->d_id) +
-			       (BBTOB(f->blf_len) / sizeof(xfs_dqblk_t)) - 1);
+			       (BBTOB(f->blf_len) / sizeof(struct xfs_dqblk)) - 1);
 		} else {
 			printf(_("	BUF DATA\n"));
 			if (!print_buffer) continue;
@@ -183,34 +183,34 @@ xlog_recover_print_buffer(
 
 STATIC void
 xlog_recover_print_quotaoff(
-	xlog_recover_item_t	*item)
+	struct xlog_recover_item *item)
 {
 	xfs_qoff_logformat_t	*qoff_f;
-	char			str[32] = { 0 };
 
 	qoff_f = (xfs_qoff_logformat_t *)item->ri_buf[0].i_addr;
+
 	ASSERT(qoff_f);
+	printf(_("\tQUOTAOFF: #regs:%d   type:"), qoff_f->qf_size);
 	if (qoff_f->qf_flags & XFS_UQUOTA_ACCT)
-		strcat(str, "USER QUOTA");
+		printf(" USER");
 	if (qoff_f->qf_flags & XFS_GQUOTA_ACCT)
-		strcat(str, "GROUP QUOTA");
+		printf(" GROUP");
 	if (qoff_f->qf_flags & XFS_PQUOTA_ACCT)
-		strcat(str, "PROJECT QUOTA");
-	printf(_("\tQUOTAOFF: #regs:%d   type:%s\n"),
-	       qoff_f->qf_size, str);
+		printf(" PROJECT");
+	printf("\n");
 }
 
 STATIC void
 xlog_recover_print_dquot(
-	xlog_recover_item_t	*item)
+	struct xlog_recover_item *item)
 {
 	xfs_dq_logformat_t	*f;
-	xfs_disk_dquot_t	*d;
+	struct xfs_disk_dquot	*d;
 
 	f = (xfs_dq_logformat_t *)item->ri_buf[0].i_addr;
 	ASSERT(f);
 	ASSERT(f->qlf_len == 1);
-	d = (xfs_disk_dquot_t *)item->ri_buf[1].i_addr;
+	d = (struct xfs_disk_dquot *)item->ri_buf[1].i_addr;
 	printf(_("\tDQUOT: #regs:%d  blkno:%lld  boffset:%u id: %d\n"),
 	       f->qlf_size, (long long)f->qlf_blkno, f->qlf_boffset, f->qlf_id);
 	if (!print_quota)
@@ -249,8 +249,10 @@ xlog_recover_print_inode_core(
 	printf(_("		uid:%d  gid:%d  nlink:%d projid:0x%04x%04x\n"),
 	       di->di_uid, di->di_gid, di->di_nlink,
 	       di->di_projid_hi, di->di_projid_lo);
-	printf(_("		atime:%d  mtime:%d  ctime:%d\n"),
-	       di->di_atime.t_sec, di->di_mtime.t_sec, di->di_ctime.t_sec);
+	printf(_("		atime:%lld  mtime:%lld  ctime:%lld\n"),
+			xlog_extract_dinode_ts(di->di_atime),
+			xlog_extract_dinode_ts(di->di_mtime),
+			xlog_extract_dinode_ts(di->di_ctime));
 	printf(_("		flushiter:%d\n"), di->di_flushiter);
 	printf(_("		size:0x%llx  nblks:0x%llx  exsize:%d  "
 	     "nextents:%d  anextents:%d\n"), (unsigned long long)
@@ -268,7 +270,7 @@ xlog_recover_print_inode_core(
 
 STATIC void
 xlog_recover_print_inode(
-	xlog_recover_item_t	*item)
+	struct xlog_recover_item *item)
 {
 	struct xfs_inode_log_format	f_buf;
 	struct xfs_inode_log_format	*f;
@@ -285,8 +287,10 @@ xlog_recover_print_inode(
 	       f->ilf_dsize);
 
 	/* core inode comes 2nd */
-	ASSERT(item->ri_buf[1].i_len == xfs_log_dinode_size(2) ||
-		item->ri_buf[1].i_len == xfs_log_dinode_size(3));
+	/* ASSERT len vs xfs_log_dinode_size() for V3 or V2 inodes */
+	ASSERT(item->ri_buf[1].i_len ==
+			offsetof(struct xfs_log_dinode, di_next_unlinked) ||
+	       item->ri_buf[1].i_len == sizeof(struct xfs_log_dinode));
 	xlog_recover_print_inode_core((struct xfs_log_dinode *)
 				      item->ri_buf[1].i_addr);
 
@@ -382,7 +386,7 @@ xlog_recover_print_icreate(
 
 void
 xlog_recover_print_logitem(
-	xlog_recover_item_t	*item)
+	struct xlog_recover_item	*item)
 {
 	switch (ITEM_TYPE(item)) {
 	case XFS_LI_BUF:
@@ -432,7 +436,7 @@ xlog_recover_print_logitem(
 
 static void
 xlog_recover_print_item(
-	xlog_recover_item_t	*item)
+	struct xlog_recover_item *item)
 {
 	int			i;
 
@@ -496,11 +500,11 @@ xlog_recover_print_item(
 
 void
 xlog_recover_print_trans(
-	xlog_recover_t		*trans,
+	struct xlog_recover	*trans,
 	struct list_head	*itemq,
 	int			print)
 {
-	xlog_recover_item_t	*item;
+	struct xlog_recover_item *item;
 
 	if (print < 3)
 		return;
