@@ -165,15 +165,13 @@ qc_count_rtblocks(
 	struct xfs_ifork	*ifp = XFS_IFORK_PTR(ip, XFS_DATA_FORK);
 	int			error;
 
-	if (!(ifp->if_flags & XFS_IFEXTENTS)) {
-		error = -libxfs_iread_extents(NULL, ip, XFS_DATA_FORK);
-		if (error) {
-			do_warn(
+	error = -libxfs_iread_extents(NULL, ip, XFS_DATA_FORK);
+	if (error) {
+		do_warn(
 _("could not read ino %"PRIu64" extents, err=%d\n"),
-				ip->i_ino, error);
-			chkd_flags = 0;
-			return 0;
-		}
+			ip->i_ino, error);
+		chkd_flags = 0;
+		return 0;
 	}
 
 	for_each_xfs_iext(ifp, &icur, &got)
@@ -222,14 +220,14 @@ quotacheck_adjust(
 	/* Count the file's blocks. */
 	if (XFS_IS_REALTIME_INODE(ip))
 		rtblks = qc_count_rtblocks(ip);
-	blocks = ip->i_d.di_nblocks - rtblks;
+	blocks = ip->i_nblocks - rtblks;
 
 	if (user_dquots)
 		qc_adjust(user_dquots, i_uid_read(VFS_I(ip)), blocks, rtblks);
 	if (group_dquots)
 		qc_adjust(group_dquots, i_gid_read(VFS_I(ip)), blocks, rtblks);
 	if (proj_dquots)
-		qc_adjust(proj_dquots, ip->i_d.di_projid, blocks, rtblks);
+		qc_adjust(proj_dquots, ip->i_projid, blocks, rtblks);
 
 	libxfs_irele(ip);
 }
@@ -254,7 +252,7 @@ qc_dquot_check_type(
 	 * expect an exact match for user dquots and for non-root group and
 	 * project dquots.
 	 */
-	if (xfs_sb_version_hascrc(&mp->m_sb) || type == XFS_DQTYPE_USER || id)
+	if (xfs_has_crc(mp) || type == XFS_DQTYPE_USER || id)
 		return ddq_type == type;
 
 	/*
@@ -308,26 +306,29 @@ qc_check_dquot(
 	if (be64_to_cpu(ddq->d_bcount) != qrec->bcount) {
 		do_warn(_("%s id %u has bcount %llu, expected %"PRIu64"\n"),
 				qflags_typestr(dquots->type), id,
-				be64_to_cpu(ddq->d_bcount), qrec->bcount);
+				(unsigned long long)be64_to_cpu(ddq->d_bcount),
+				qrec->bcount);
 		chkd_flags = 0;
 	}
 
 	if (be64_to_cpu(ddq->d_rtbcount) != qrec->rtbcount) {
 		do_warn(_("%s id %u has rtbcount %llu, expected %"PRIu64"\n"),
 				qflags_typestr(dquots->type), id,
-				be64_to_cpu(ddq->d_rtbcount), qrec->rtbcount);
+				(unsigned long long)be64_to_cpu(ddq->d_rtbcount),
+				qrec->rtbcount);
 		chkd_flags = 0;
 	}
 
 	if (be64_to_cpu(ddq->d_icount) != qrec->icount) {
 		do_warn(_("%s id %u has icount %llu, expected %"PRIu64"\n"),
 				qflags_typestr(dquots->type), id,
-				be64_to_cpu(ddq->d_icount), qrec->icount);
+				(unsigned long long)be64_to_cpu(ddq->d_icount),
+				qrec->icount);
 		chkd_flags = 0;
 	}
 
 	if ((ddq->d_type & XFS_DQTYPE_BIGTIME) &&
-	    !xfs_sb_version_hasbigtime(&mp->m_sb)) {
+	    !xfs_has_bigtime(mp)) {
 		do_warn(
 	_("%s id %u is marked bigtime but file system does not support large timestamps\n"),
 				qflags_typestr(dquots->type), id);
@@ -438,15 +439,13 @@ quotacheck_verify(
 	}
 
 	ifp = XFS_IFORK_PTR(ip, XFS_DATA_FORK);
-	if (!(ifp->if_flags & XFS_IFEXTENTS)) {
-		error = -libxfs_iread_extents(NULL, ip, XFS_DATA_FORK);
-		if (error) {
-			do_warn(
+	error = -libxfs_iread_extents(NULL, ip, XFS_DATA_FORK);
+	if (error) {
+		do_warn(
 	_("could not read %s inode %"PRIu64" extents, err=%d\n"),
-				qflags_typestr(type), ip->i_ino, error);
-			chkd_flags = 0;
-			goto err;
-		}
+			qflags_typestr(type), ip->i_ino, error);
+		chkd_flags = 0;
+		goto err;
 	}
 
 	/* Walk each extent of the quota inode and compare counters. */
